/* Cyclic Redundancy Check algorithm, complete with traditional error.
 * Copyright  2002, 2003, 2004 Ted Phelps
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms fo the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the `etagit' program; see the file `COPYING'.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place -
 * Suite 330, Boston, MA 02111-1307, USA. */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#ifdef HAVE_WINCONFIG_H
#include <winconfig.h>
#endif
#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif
#ifdef HAVE_STDLIB_H
# include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

/* The argument to the current option */
char *optarg = NULL;

/* Our pointer into an args options */
static char *optp = NULL;

/* The index into argv of the next argument to be processed */
int optind = 1;

/* Whether or not to print a message when an error occurs */
int opterr = 1;

/* The value of the character which caused the error */
int optopt = 0;

/* The getopt function itself */
int getopt(int argc, char *const *argv, const char *optstring)
{
    char *point;
    int ch;

    /* Assume no option argument until we determine otherwise */
    optarg = NULL;

    /* Are we out of args? */
    if (! (optind < argc)) {
        optp = NULL;
        return EOF;
    }

    /* Is this the first time we've looked at the arg? */
    if (! optp) {
        /* Yes.  Let's see if it's an option */
        optp = argv[optind];

        /* If it doesn't begin with a dash then we're done */
        if (*optp != *OPTION_PREFIX) {
            optp = NULL;
            return EOF;
        }

        /* Move on to the next character */
        optp++;

        /* A single dash by itself isn't an option */
        if (*optp == '\0') {
            optp = NULL;
            return EOF;
        }

        /* Two dashes are the end of options marker */
        if (strcmp(optp, "-") == 0) {
            optind++;
            optp = NULL;
            return EOF;
        }
    }

    /* Get the option character */
    ch = *(optp++);

    /* Is it in the specification string? */
    if (! (point = strchr(optstring, ch))) {
        if (opterr) {
            fprintf(stderr, "%s: illegal option -- %c\n", argv[0], ch);
        }

        optopt = ch;
        return '?';
    }

    /* Does it require an argument? */
    if (point[1] != ':') {
        /* No.  Arrange to read the next arg on the next call to getopt. */
        if (*optp == '\0') {
            optind++;
            optp = NULL;
        }

        return ch;
    }

    /* Point optarg at the option's argument. */
    if (*optp == '\0') {
        optind++;
        optp = NULL;

        /* Make sure we're not out of arguments */
        if (! (optind < argc)) {
            fprintf(stderr, "%s: option requires an argument -- -%c\n", argv[0], ch);
            optopt = ch;
            return '?';
        }

        optarg = argv[optind++];
    } else {
        optarg = optp;
        optp = NULL;
        optind++;
    }

    return ch;
}

#ifdef DEBUG_REPLACE
int main(int argc, char *argv[])
{
    int ch;

    /* Go through the options and their args */
    while ((ch = getopt(argc, argv, "a:c:C:e:f:h:H:i:p:mn:s:k:S:i:qvx")) > 0) {
        printf("option: %c, arg: %s\n", ch, optarg ? optarg : "<none>");
    }

    /* Print out the command args */
    while (optind < argc) {
        printf("arg: %s\n", argv[optind++]);
    }

    exit(0);
}
#endif /* DEBUG_REPLACE */

